from omegaconf import OmegaConf,DictConfig
from pathlib import Path
import time
from functools import lru_cache
import importlib

__all__=['measure_time','load_config','load_class','load_algorithm_class','load_problem_class']


def measure_time(func):
    def wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        result = func(*args, **kwargs)
        end_time = time.perf_counter()
        run_time = end_time - start_time
        return run_time,result
    return wrapper

def load_config(fname:Path)->DictConfig:
    '''
    从fname指定的配置文件加载配置参数
    '''
    cfg = OmegaConf.load(fname)
    return cfg

@lru_cache(maxsize=None)  # maxsize=None表示无限制缓存大小
def load_class(full_name:str):
    '''
    动态加载指定的类,如果已经加载过，直接从缓存返回
    '''    
    try:
        # 使用importlib动态加载模块
        cls_name=full_name.split('.')[-1]
        module = importlib.import_module(f'{full_name.lower()}')
        return getattr(module, cls_name)
    except ImportError:
        print(f"Module {full_name} not found.")
        return None

def load_algorithm_class(name:str,package_name:str=None):
    '''
    动态加载算法类
    '''
    if  package_name is None:
        package_name='siaf.algorithms'     
    return load_class(f'{package_name}.{name}')

def load_problem_class(name:str,package_name:str=None):
    '''
    动态加载算法类
    ''' 
    if  package_name is None:
        package_name='siaf.problems'  
    return load_class(f'{package_name}.{name}')


import vrplib
from pathlib import Path
from typing import List
import numpy as np
from siaf.utils.common import load_config

def test_read_data():
    cfg_path=Path(__file__).parent.parent/'configs/app.yaml'
    app_cfg=load_config(cfg_path)
    
    assert app_cfg.NUM_ITRATION>1000 and app_cfg['NUM_REPEAT']>1



from numpy.typing import NDArray
from .base import BaseSIA
import numpy as np
class BBPSO(BaseSIA):
    def __init__(self, particles=20,step=1,best_weight=0.5):
        super().__init__(particles)
        self.step=step
        self.best_weight=best_weight

    def new_positions(self,t:int)->NDArray:
        w=self.best_weight
        noises=np.random.randn(self.num_individuals,len(self.best_x))
        return w*self.best_x*+(1-w)*self.best_xs+self.step*noises
    


from numpy.typing import NDArray
from .bbpso import BBPSO
import numpy as np
from typing import Dict
from ..problems import IProblem

class LSO(BBPSO):
    def __init__(self, particles=20,step=0.5,best_weight=0.5,num_adult=5 ):
        super().__init__(particles,step,best_weight)
        assert 1<=num_adult<=particles//2
        self.num_adult=num_adult
       
    def reset(self,problem:IProblem):
        super().reset(problem)
        self.stat:Dict[str,int]={'King':0,'Lioness':0,'lion_cub':0}

    def new_positions(self,t:int)->NDArray:
        w=self.best_weight
        T=self.num_iterations
        scale=(T-t)/T
        n,d=self.num_individuals,len(self.best_x)
        noises=np.random.randn(n,d)
        xs=w*self.best_x+(1-w)*self.best_xs+self.step*scale*noises
        for i,idx in enumerate(self.sort_indexs):
            noise=np.random.randn(d)
            if i==0:
                K=0.8
                xs[idx]=K*self.best_x+(1-K)*self.best_xs[idx]*+self.step*0.01*scale*noise
            elif i<self.num_adult:
                co_id=np.random.randint(1,self.num_adult)
                co_idx=self.sort_indexs[co_id]
                K=0.618
                xs[idx]=K*self.best_xs[idx]+(1-K)*self.best_xs[co_idx]+0.2*scale*noise
        return xs
    
    def record_best(self,t=0,index=0):
        i=np.argwhere(self.sort_indexs==index)[0]
        flag='lion_cub'
        if i==0:
            flag='King'
        elif i>self.num_adult:
            flag='lion_cub'
        else:
            flag='Lioness'
        self.stat[flag]+=1
        print(f"step:{t+1} {flag} find new best:{self.best_cost}")




from pathlib import Path
INSTANCE_ROOT=Path(__file__).parent.parent.parent/'instances'
EXPERIMENT_ROOT=Path(__file__).parent.parent.parent/'outputs/experiments'

def get_files(dir='cvrp',suffix=None,flag='instance'):
    if 'instance'==flag:
        data_dir:Path=INSTANCE_ROOT/dir
    elif 'experiment'==flag:
        data_dir=EXPERIMENT_ROOT/dir
    else :
        data_dir=Path(dir)
    rt=[]
    for item in data_dir.iterdir():
        if item.is_file() and (item.suffix is None or item.suffix==suffix): 
            rt.append(item)
        elif item.is_dir():
            rt.extend(get_files(dir+f'/{item.stem}',suffix))
    return rt 
     



from abc import ABC, abstractmethod
from typing import Tuple
from numpy.typing import NDArray

# 定义一个问题接口
class IProblem(ABC):
    
    @property
    def case_name(self)->str:
        pass

    @property
    def shape(self)->Tuple:
        pass
    
    @abstractmethod
    def cost(self,x:NDArray)->float:
        pass
    
    def show(self,x:NDArray,title='demo',suff:str=''):
        print(x)


from numpy.typing import NDArray
from .bbpso import BBPSO
import numpy as np
class APSO(BBPSO):

    def new_positions(self,t:int)->NDArray:
        w=self.best_weight
        scale=t/self.num_iterations
        noises=np.random.randn(len(self.best_x))
        return w*self.best_x*+(1-w)*self.best_xs+self.step*scale*noises



import vrplib
from pathlib import Path
from typing import List
import numpy as np
from siaf.utils.data import *

def test_fill_text_data():
    lines=[['1','2'],['1','2','3']]
    data=textlines2ndarray(lines)
    assert data.shape==(2,3)

def test_extend_data():
    lines1=[['1','2'],['1','2','3']]
    lines2=[['4','5','6'],['6','7','8','9']]
    data1=textlines2ndarray(lines1)
    data2=textlines2ndarray(lines2)
    assert data1.shape==(2,3)
    assert data2.shape==(2,4)
    data=extend_numpy_list([data1,data2])
    assert 2==len(data)
    assert data[0].shape==data[1].shape==(2,4)
    d1=data[0][0]
    assert d1[2]==d1[3]==2
    d2=data[1][0]
    assert d2[3]==6


import numpy as np
from numpy.typing import NDArray

from .problem import IProblem

class X2(IProblem):
    def __init__(self, dim=3):
        self.dim =dim

    @property
    def shape(self):
        return (self.dim,)

    @property
    def case_name(self)->str:
        return f'{self.__class__.__name__}-{self.dim}' 
      
    def cost(self,x:NDArray)->float:
        return np.linalg.norm(x, axis=-1)

if __name__ == '__main__':
    f=X2(2)
    fv=f.cost(np.array([3.0,4.0]))
    print(fv) 
    assert(fv==5.0) #


import matplotlib.pyplot as plt
from pathlib import Path
from typing import List
from numpy.typing import NDArray
from .files import get_files
from .data import load_data,get_stat,get_distance,extend_numpy_list
__all__=['draw_routes','draw_lines']
default_colors = plt.rcParams['axes.prop_cycle'].by_key()['color']  
my_font = {
    'family': 'Times New Roman',
    'weight': 'normal',
    'size': 18,
}

def draw_routes(coordinates:NDArray,demands:NDArray,distance_matrix:NDArray,routes,title:str,suff:str):
    fig = plt.figure(figsize=(10, 8))
    ax= fig.add_subplot(1, 1,1 )
    ax.set_title(title)
    plt.ioff()
    draw_nodes(ax,coordinates,demands)
    print('-'*60)
    ds=[]
    for i,route in enumerate(routes):
        route=[0]+route+[0] if route[0]!=0 else route
        dis=get_distance(route,distance_matrix)
        ds.append(dis)
        rs=demands[route] if demands is not None else []
        print(f'route#{i+1} distance:{dis:.1f} {route} demand:{sum(rs)} {rs}')
        draw_route(ax,coordinates,demands,distance_matrix,route,default_colors[i])
    print('-'*60)
    print(f'total:{sum(ds)} distance:{ds}')
    title=''.join(title.split())
    plt.savefig(f'outputs/{title}_{suff}.jpg')
    plt.pause(4)

def draw_nodes(ax,coordinates:NDArray,demands:NDArray=None):
    ax.scatter(coordinates[:, 0], coordinates[:, 1], color='blue', s=60, marker='o')
    # 添加城市名称
    for i, pos in enumerate(coordinates):
        if i==0 :
            continue
        d=demands[i] if demands is not None else 0
        info=f'[{d:.1f}]' if d>0 else ''
        plt.text(pos[0], pos[1]+0.3, str(i)+info , fontsize=12, ha='center', va='bottom')

def draw_route(ax,coordinates:NDArray,demands:NDArray,distance_matrix:NDArray,route,color='green'):
    # 绘制访问路径
    ds=[]
    for i in range(len(route) - 1):
        ax.plot([coordinates[route[i], 0], coordinates[route[i + 1], 0]],
                [coordinates[route[i], 1], coordinates[route[i + 1], 1]], color=color, linestyle='-', linewidth=2)

    # 连接最后一个城市和起始城市
    ax.plot([coordinates[route[-1], 0], coordinates[route[0], 0]],
            [coordinates[route[-1], 1], coordinates[route[0], 1]], color=color, linestyle='-', linewidth=2)
     

def add_opt_line(ax,opt_name,opt_data,index=0,fill_between=True):
    iters=range(1,1+len(opt_data))
    #color = palette(index)  #算法颜色
    avg = opt_data[:,0]
    std = opt_data[:,1]
    r1 = list(map(lambda x: x[0] - x[1], zip(avg, std)))  #上方差
    r2 = list(map(lambda x: x[0] + x[1], zip(avg, std)))  #下方差
    ax.plot(iters, avg,  label=opt_name, linewidth=3.0)
    if fill_between:
        ax.fill_between(iters, r1, r2, alpha=0.06)


def draw_lines(case_dir:str,title='',fill_between=True,ax=None):
    fs:List[Path]=get_files(case_dir,'.txt','experiment')
    if not ax:
        fig = plt.figure(figsize=(10, 6))
        ax= fig.add_subplot(1, 1,1 )
        plt.ioff()
    idx=0
    names=[]
    data=[]
    for f in fs:
        names.append(f.stem)
        d=load_data(f)
        data.append(d)
    data=extend_numpy_list(data)
    d2=[]
    for d in data:    
        d2.append(get_stat(d))

    for name,opt_data in zip(names,d2):
        add_opt_line(ax,name,opt_data,idx,fill_between)
        idx+=1

    ax.set_title(title)
    ax.legend(loc='lower right', prop=my_font)
    ax.set_xlabel('iterations', fontsize=22)
    ax.set_ylabel('cost', fontsize=22)
    plt.savefig(f'outputs/{title}.jpg')
    plt.pause(4)
    #plt.show()





from typing import List,Tuple
from numpy.typing import NDArray
import numpy as np
from .problem import IProblem
from ..utils.plot import draw_routes
from ..utils.data import get_distance
class TSP(IProblem):
    def __init__(self, num_city=30):
        self.num_city = num_city
        data=self._make_random_data(num_city)
        self.city_coords:NDArray=data[0]
        self.distance_matrix:NDArray=data[1]

    @property
    def case_name(self)->str:
        return f'{self.__class__.__name__}-{self.num_city}' 
    
    @property
    def shape(self):
        return (self.num_city,)

       
    def cost(self,x:NDArray)->float:
        '''
        x--每个城市的选中优先级系数(0~1),越小越排前
        '''
        tour:List=np.argsort(x).tolist()
        tour=self._shift(tour,0)
        tour+=[0]
        return get_distance(tour,self.distance_matrix)

    def show(self,x:NDArray,title='demo',suff:str=''):
        route:List=np.argsort(x).tolist()
        route=self._shift(route,0)
        draw_routes(self.city_coords,None,self.distance_matrix,[route],title,suff)
        

    def _shift(self,tour:List,start=0)->List[int]:
        index=tour.index(start)
        return tour[index:]+tour[0:index]
    
    def _make_distance_matrix(self,city_coords:NDArray)->NDArray:
        '''

            广州 福州 武汉 北京
        广州   0   100  200 300
        福州   100 0    250 280
        武汉   200 250  0   280
        北京   300 280  280 0


        # N=len(pos)
        # rt=np.ones((N,N),dtype=float)*1e10
        # for i in range(N):
        #     temp=pos-pos[i]
        #     temp=temp**2
        #     temp=np.sqrt(temp.sum(axis=1))
        #     idxs=temp>0.1
        #     rt[i][idxs]=temp[idxs]

        # rt=np.sqrt(
        #     ((pos[:, :, None] - pos[:, :, None].T) ** 2).sum(axis=1)
        # )

        '''
        expanded_coords = np.expand_dims(city_coords, axis=1)  # 
        relative_positions = expanded_coords - expanded_coords.transpose((1, 0, 2))  # 计算相对位置矩阵  
        distances = np.linalg.norm(relative_positions, axis=2)  # 计算欧几里德距离    
        return np.round(distances)

    def _make_random_data(self,N=5,low=-10,high=10)->Tuple[NDArray,NDArray]:
        '''
        元组第一个数据是坐标，第二个是距离矩阵
        '''
        while True:
            city_coords=np.random.randint(low,high,(N,2))
            dis_matrix=self._make_distance_matrix(city_coords)
            idxs=dis_matrix<1e-10
            #       [[True False ....]
            #        [False True ....]
            #                         ]
            dis_matrix[idxs]=1e10
            if np.sum(dis_matrix==1e10)==N:
                break
        return city_coords,dis_matrix


from numpy.typing import NDArray
from typing import Union,List
from pathlib import Path
import numpy as np

def get_distance(tour:List[int],distance_matrix:NDArray)->int:
    if tour[0]!=tour[-1]:
        tour=[0]+tour+[0]
    # total=0
    # for i in range(len(tour)-1):
    #     total+=distance_matrix[tour[i],tour[i+1]]
    idxs1 = tour
    idxs2 = tour[1:] + tour[:1]
    return np.round(distance_matrix[idxs1, idxs2].sum())

  
def get_stat(data:NDArray)->NDArray:
    '''
    生成实验数据的统计信息
    '''
    steps=len(data[0])
    rt=np.zeros((steps,2))
    for i in range(steps):
        cols=data[:,i]
        rt[i,:]=[np.mean(cols),np.std(cols)]
    return rt

def normalize_coord(coord:NDArray) -> NDArray:
    '''
    对2维坐标数组进行归一化处理
    '''
    x, y = coord[:, 0], coord[:, 1]
    x_min, x_max = x.min(), x.max()
    y_min, y_max = y.min(), y.max()
    
    x_scaled = (x - x_min) / (x_max - x_min) 
    y_scaled = (y - y_min) / (y_max - y_min)
    coord_scaled = np.stack([x_scaled, y_scaled],axis=1)
    return coord_scaled 

def textlines2ndarray(lines:List[List[str]])->NDArray:
    '''
    对2维列表中进行填充，使得各行的长度相同
    '''
    # 找到数据最多的行  
    max_data_line = max(lines, key=len)  
    rows=[list(max_data_line)]*len(lines)

    # 将数据行转为 NumPy 数组  
    rt = np.array(rows, dtype=float)
    for i,line in enumerate(lines):
        row_data=np.array(line, dtype=float)
        rt[i,:len(row_data)]=row_data
        rt[i,len(row_data):]=row_data[-1]
    return rt

def extend_numpy_list(data:List[NDArray])->List[NDArray]:
    '''
    对列表中numpy数据进行填充，使得各行的长度相同
    '''
    # 找到数据最多的行 
    dim=len(data[0].shape)
    num_cols=map(lambda d:len(d[0]),data) if dim>1 else map(lambda d:len(d),data) 
    max_num =max(num_cols) 
    rt=[]
    for ds in data:
        if dim>1:
            expanded_data = np.pad(ds, ((0, 0), (0,max_num-len(ds[0]))), 'edge') 
        else:
            expanded_data = np.pad(ds, (0,max_num-len(ds)), 'edge') 
        rt.append(expanded_data) 
    return rt

def load_data(fpath:Union[str,Path])->NDArray:
    data_lines = []  
    # with  open(fpath) as f:
    #     data=np.loadtxt(f, delimiter=",")
    # return data
    with open(fpath, 'r') as file:  
        for line in file:  
            if line.startswith('#'):
                continue
            # 清洁并分割数据  
            data = line.strip().split(',')  
            data_lines.append(data)  
    return textlines2ndarray(data_lines)






from siaf.utils.files import get_files

def test_read_cvrp_files():
    fs=get_files('cvrp/demo','.vrp')
    fname=str(fs[0])
    ss=fname.split('\\')
    assert len(ss)>3
def test_experiments():
    fs=get_files('case1','.txt','experiment')
    assert 3==len(fs)




from ..problems import IProblem
from functools import reduce 
import numpy as np
from numpy.typing import NDArray
from typing import List
import operator  
class BaseSIA:
    def __init__(self, num_individuals=20):
        self.num_individuals = num_individuals
        self.problem=None

    @property
    def name(self)->str:
        return self.__class__.__name__
    
    def run(self, iterations=1000):
        self.num_iterations:int=iterations
        num_quit=iterations//3
        self.cnt=0
        records:List[float]=[]
        for t in range(iterations):
            self.iterate(t)
            self.cnt+=1
            records.append(self.best_cost)
            if self.cnt>num_quit:
                break 
        self.records:NDArray=np.array(records)
        return self.best_cost,self.best_x
        
    def reset(self,problem:IProblem):
        self.problem:IProblem=problem
        num_weights=reduce(operator.mul, problem.shape, 1) 
        self.xs:NDArray=np.random.randn(self.num_individuals,num_weights)
        costs=np.apply_along_axis(problem.cost, axis=1, arr=self.xs)
        self.sort_indexs=np.argsort(costs)
        min_idx=self.sort_indexs[0]
        self.best_xs:NDArray=self.xs.copy()
        self.best_costs:NDArray= costs.copy()
        self.best_x:NDArray=self.xs[min_idx].copy()
        self.best_cost:float=costs[min_idx]
        

    def check_best(self,t:int,xs:NDArray,costs:NDArray)->bool:
        idxs= costs<self.best_costs
        #self.xs[:,:]=xs[:,:]
        self.best_xs[idxs,:]=xs[idxs,:]
        self.best_costs[idxs]=costs[idxs]
        min_idx=np.argmin(costs)
        if (costs[min_idx]<self.best_cost):
            self.best_cost=costs[min_idx]
            self.best_x=xs[min_idx]
            self.record_best(t,min_idx)
            self.sort_indexs=np.argsort(costs)
            self.cnt=0
            return True
        return False

    def iterate(self,t:int):
        xs=self.new_positions(t)
        costs=np.apply_along_axis(self.problem.cost, axis=1, arr=xs)
        if not self.check_best(t,xs,costs) and t%3==2:
            self.sort_indexs=np.argsort(costs)

    def record_best(self,t=0,index=0):
        print(f"step:{t+1} agent{index+1} find best:{self.best_cost}")

    def new_positions(self,t:int)->NDArray:
        pass


from numpy.typing import NDArray
from .base import BaseSIA
import numpy as np
from ..problems import IProblem
class PSO(BaseSIA):
    def __init__(self, particles=20,inertia=0.5,cognitive_coeff=1.5, social_coeff=1.5):
        super().__init__(particles)
        self.cognitive_coeff=cognitive_coeff
        self.social_coeff=social_coeff
        self.inertia=inertia

    def reset(self,problem:IProblem):
        super().reset(problem)
        self.vs= np.random.uniform(-1, 1, size=self.best_x.shape)

    def new_positions(self,t:int)->NDArray:
        r1, r2 = np.random.rand(2)  
        cognitive = self.cognitive_coeff * r1 * (self.best_xs - self.xs)  
        social = self.social_coeff * r2 * (self.best_x - self.xs)    
        noise = np.random.normal(0, 0.1, size=self.vs.shape)  # 添加噪声  
        self.vs = self.inertia * self.vs + cognitive + social + noise  
        self.xs += self.vs  
        return self.xs
        # 确保粒子在边界内  
        #self.xs = np.clip(self.xs, bounds[:, 0], bounds[:, 1])  
       





from setuptools import setup, find_packages  

setup(  
    name='siaf',  
    version='0.1.1',    
    packages=find_packages(),    
    install_requires=[
        'numpy>=1.26.4',
        'pandas>=2.2.2',  
        'matplotlib>=3.8.0', 
        'omegaconf>=2.3.0',
        'vrplib>=1.4.0',
        'pyDOE2>=1.3.0'
    ],  
    author='alex',  
    author_email='lsj178@139.com',  
    description='Swarm Intelligence Algorithm Framework',  
    long_description=open('README.md').read(),    
    long_description_content_type='text/markdown',  
    url='https://github.com/lshengjian/siaf',    
    classifiers=[  
        'Programming Language :: Python :: 3',  
        'License :: OSI Approved :: MIT License',  
        'Operating System :: OS Independent',  
    ],  
    python_requires='>=3.8',  
)


import pyDOE2
from typing import List
from numpy.typing import NDArray
__all__=['make_design']
def make_design(levels:List[int],reduction=2)->NDArray:
    return pyDOE2.gsd(levels, reduction) # 从2^3组合中抽1/2来实验，如果是3则为1/3

if __name__ == '__main__':
    import pandas as pd
    # 模拟执行试验并记录结果
    results = []
    for i, row in enumerate(make_design([2,2,2])):
        # 模拟执行试验，这里用随机数代替
        result = {
            'Factor1': row[0],
            'Factor2': row[1],
            'Factor3': row[2],
            'Result': i * 10  # 模拟结果
        }
        results.append(result)

    # 将结果保存到 DataFrame
    df = pd.DataFrame(results)
    print(df)


import itertools
import numpy as np


def test_combination():
    data=list(itertools.product([0, 1], repeat=3))
    assert 8==len(data) and (0,0,0)==data[0] and   (1,1,1)==data[7]


def test_nonzero():
    idxs=np.nonzero([0,1.5,2])[0] # olny one dimetion
    assert [1,2]==idxs.tolist()

    idx=np.argwhere(np.array([0,2,1,5,3],dtype=int)==5)[0]
    assert idx==3

def test_trim():
    msg='hello word'
    assert 'helloword'==''.join(msg.split()) 


from typing import List,Tuple
from numpy.typing import NDArray
import numpy as np
import matplotlib.pyplot as plt
import vrplib
from .problem import IProblem
from pathlib import Path
from ..utils.files import INSTANCE_ROOT
from ..utils.plot import draw_routes
from ..utils.data import get_distance
class CVRP(IProblem):
    def __init__(self, num_city=30,capacity=1,fname:str=None):
        data=[]
        if fname!=None:
            if not fname.startswith(str(INSTANCE_ROOT)):
                self.fname=INSTANCE_ROOT/f'cvrp/{fname}'
            else:
                self.fname=Path(fname)
            data=self._load_data()
        else:
            self.fname=None
            self.num_city = num_city
            self.capacity=capacity
            data=self._make_random_data(num_city)
       
        self.city_coords:NDArray=data[0]
        self.distance_matrix:NDArray=data[1]
        self.demands=data[2]

    @property
    def case_name(self)->str:
        cls_name=self.__class__.__name__
        rt=f'{cls_name}-{self.num_city}' if self.fname is None else f'{cls_name}-{self.fname.stem}'
        return  rt
    @property
    def shape(self):
        return (self.num_city,)

    def _make_solution(self,x:NDArray)->List[List[int]]:
        visits:List=np.argsort(x)+1# 0 是仓库
        tour=[0]
        result:List[List[int]]=[]
        total_demand=0
        for id in visits:
            last=tour[-1]
            dx=self.distance_matrix[last,id]
            temp=total_demand+self.demands[id]
            if temp>self.capacity:
               result.append(tour+[0])
               tour=[0,id] 
               total_demand=self.demands[id] 
            elif temp<self.capacity:
                tour.append(id)
                total_demand=temp
            else:
               result.append(tour+[id,0])
               total_demand=0
               tour=[0]


        if tour[-1]!=0:
            last=tour[-1]
            tour.append(0)
            result.append(tour)
        return result

    def cost(self,x:NDArray)->float:
        routes=self._make_solution(x)
        total=0
        for route in routes:
            total+=get_distance(route,self.distance_matrix)
        return total
    
    def show_solution(self,routes:List[List[int]],title='demo',suff='_sol'):
        draw_routes(self.city_coords,self.demands,self.distance_matrix,routes,title,suff)

    def show(self,x:NDArray,title='demo',suff:str=''):
        routes=self._make_solution(x)
        draw_routes(self.city_coords,self.demands,self.distance_matrix,routes,title,suff)
    
    def _make_distance_matrix(self,city_coords:NDArray)->NDArray:
        expanded_coords = np.expand_dims(city_coords, axis=1)  # 
        relative_positions = expanded_coords - expanded_coords.transpose((1, 0, 2))  # 计算相对位置矩阵  
        distances = np.linalg.norm(relative_positions, axis=2)  # 计算欧几里德距离    
        return np.round(distances)
    
    def _load_data(self)->Tuple:
        
        problem = vrplib.read_instance(self.fname)
        fp=self.fname.parent/f'{self.fname.stem}.sol'
        self.solution =vrplib.read_solution(fp)
        
        self.capacity=problem['capacity']
        coords = np.array(problem['node_coord'],dtype=float)
        self.num_city = coords.shape[0]-1
        reqs=np.array(problem['demand'],dtype=float)
        dis_matrix=self._make_distance_matrix(coords)
        return coords,dis_matrix,reqs


    def _make_random_data(self,N=5,low=-10,high=10)->Tuple:
        '''
        元组第一个数据是坐标，第二个是距离矩阵，最后是需求数据
        '''
        while True:
            city_coords=np.random.randint(low,high,(N+1,2))# 0--仓库
            reqs=np.random.rand(N+1) # 0-备用
            dis_matrix=self._make_distance_matrix(city_coords)
            idxs=dis_matrix<1e-10
            #       [[True False ....]
            #        [False True ....]
            #                         ]
            dis_matrix[idxs]=1e10
            if np.sum(dis_matrix==1e10)==N+1:
                break
        return city_coords,dis_matrix,reqs